/*
 * Segment 10 System (Sys), SWE 10.1 StartStopp
 * Copyright (C) 2007-2017 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.sys.startstopp.console.ui.online;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.googlecode.lanterna.gui2.dialogs.ActionListDialog;
import com.googlecode.lanterna.gui2.dialogs.ActionListDialogBuilder;

import de.bsvrz.sys.startstopp.api.jsonschema.Applikation;
import de.bsvrz.sys.startstopp.api.jsonschema.Inkarnation;
import de.bsvrz.sys.startstopp.console.ui.EditableTable;

class OnlineInkarnationTable extends EditableTable<Applikation> {

	OnlineInkarnationTable(List<Applikation> applikationen) {
		super(applikationen, "Inkarnation", "Art", "Status", "Zeit", "Starts", "Meldung");
		setTableCellRenderer(new OnlineTableCellRenderer());
	}

	public void updateApplikationen(List<Applikation> applikationen) {
		String selectedName = getInkarnationsName(getSelectedElement());
		clearTable();
		int selectedRow = -1;
		int row = 0;
		for (Applikation applikation : applikationen) {
			if (selectedName.equals(getInkarnationsName(applikation))) {
				selectedRow = row;
			}
			addElement(applikation);
			row++;
		}

		if (selectedRow >= 0) {
			setSelectedRow(selectedRow);
		}
	}

	private String getInkarnationsName(Applikation selectedElement) {
		String name = null;
		if(selectedElement != null) {
			Inkarnation inkarnation = selectedElement.getInkarnation();
			if(inkarnation != null) {
				 name = inkarnation.getInkarnationsName();
			}
		}

		if(name == null) {
			return "";
		}
		return name;
	}

	@Override
	protected Applikation requestNewElement() {
		return null;
	}

	@Override
	protected Applikation editElement(Applikation applikation) {
		ActionListDialogBuilder builder;
		if (applikation != null) {
			builder = new ActionListDialogBuilder().setTitle("Applikation").setCanCancel(false);
			builder.addAction(new ApplikationStartAction(applikation));
			builder.addAction(new ApplikationRestartAction(applikation));
			builder.addAction(new ApplikationStoppAction(applikation));
			builder.addAction(new ApplikationDetailAction(applikation));
			builder.addAction(new ApplikationLogAction(applikation));
			builder.setDescription("ESC - Abbrechen");
			ActionListDialog dialog = builder.build();
			dialog.setCloseWindowWithEscape(true);
			dialog.showDialog(getTextGUI());
		}
		return null;
	}

	@Override
	protected List<String> getStringsFor(Applikation applikation) {
		List<String> result = new ArrayList<>();
		if(applikation.getGeaendert()) {
			result.add("!" + applikation.getInkarnation().getInkarnationsName());
		} else {
			result.add(applikation.getInkarnation().getInkarnationsName());
		}
		result.add(getStartArtKurzname(applikation));
		result.add(applikation.getStatus().toString());
		result.add(getStartStopInfo(applikation));
		result.add(getStartCounts(applikation));
		result.add(applikation.getStartMeldung());
		return result;
	}

	private String getStartCounts(Applikation applikation) {
		Integer anzahlStarts = applikation.getAnzahlStarts();
		if (anzahlStarts == null) {
			return "";
		}
		return anzahlStarts.toString();
	}

	private String getStartStopInfo(Applikation applikation) {
		switch (applikation.getStatus()) {
		case GESTARTET:
		case INITIALISIERT:
		case STOPPENWARTEN:
			if (applikation.getLetzteStartzeitSeconds() != null) {
				return String.format("A: %s", new SimpleDateFormat("dd.MM.yy HH:mm")
						.format(new Date(applikation.getLetzteStartzeitSeconds() * 1000L)));
			}
			break;

		case GESTOPPT:
		case STARTENWARTEN:
		case INSTALLIERT:
			if (applikation.getLetzteStoppzeitSeconds() != null) {
				return String.format("E: %s", new SimpleDateFormat("dd.MM.yy HH:mm")
						.format(new Date(applikation.getLetzteStoppzeitSeconds() * 1000L)));
			}
			break;

		}
		return "";
	}

	private String getStartArtKurzname(Applikation applikation) {

		String result;

		switch (applikation.getInkarnation().getStartArt().getOption()) {
		case AUTOMATISCH:
			result = "AUT";
			break;
		case INTERVALLABSOLUT:
			result = "ABS";
			break;
		case INTERVALLRELATIV:
			result = "REL";
			break;
		case MANUELL:
			result = "MAN";
			break;
		default:
			result = "???";
			break;
		}

		return result;
	}

	@Override
	protected boolean checkDelete(Applikation element) {
		return false;
	}
}